package VocTrainer::Database;


###########################################################################
#
#  a part of "Weird Web v0.2" a web publishing system (framework)
#  Copyright (C) 2002 koppi@weirdoz.net
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#  
###########################################################################


require Exporter;
use vars qw(@ISA @EXPORT %conf);
@ISA = qw(Exporter);
@EXPORT = qw(
	&DB_open 
	&DB_close 
	&DB_getdata 
	&DB_insertdata 
	&DB_commit 
	&DB_rollback 
	&DB_getlastinsertid
);

use strict;

use VocTrainer::Utils;
use VocTrainer::Config;

use constant DEBUG => 0;

use vars(%conf);
%conf = VocTrainer::Config::returnconf();

use DBI;

1;

=head1 NAME

VocTrainer::Database - Weird Web Wrapper for DBI

=head1 SYNOPSIS

 use VocTrainer::Database;

 my $dbh = DB_open();
 @data = @{DB_getdata(STMT => 'sql-stmt', DBH => $dbh)};
 DB_insertdata (STMT => 'sql-stmt', DBH => $dbh);
 my $lastid = DB_getlastinsertid(DBH => $dbh);
 DB_close(DBH => $dbh);


=head1 DESCRIPTION

 Should support mysql (will maybe support oracle and sybase)
 Requires VocTrainer::Config, VocTrainer::Utils
 All functions are exported

=head1 FUNCTIONS


=over 4

=cut


####
#### db routines
####

=item B<DB_open>

Connect to database and return DBI - DatabaseHandle.
No Params required if defined in VocTrainer::Config

B<Params:>
 DB => 'mysql'            (only mysql yet)
 DBNAME => $conf{dbname}  (name of db)
 DBUSER => $conf{dbuser}  (db username)
 DBPASS => $conf{dbpass}  (db passwd)

=cut

sub DB_open
{
	my %args = (DB => 'mysql',
				DBNAME => $conf{dbname},
				DBUSER => $conf{dbuser},
				DBPASS => $conf{dbpass},
				@_
				);

    my $dsn;
    my $dbh;
	$dsn = "dbi:mysql:database=$args{DBNAME}$conf{mysqloptions}";
	$dbh = DBI->connect($dsn, $args{DBUSER}, $args{DBPASS}, { RaiseError => 1 });
	#					{ RaiseError => 1, AutoCommit => 0 });
	if (!$dbh) {
		VocTrainer::Utils::handle_error ("DB: ",$DBI::errstr);
	}
    return $dbh;
}


=item B<DB_close>

Disconnect a open DBH.

B<Required Params:>
 DBH => $DBI::Object (Database Handle)

=cut

sub DB_close
{
	my %args = (DBH => '',
				@_);
	return unless $args{DBH};
    my $rc = $args{DBH}->disconnect || handle_warning ( $args{DBH}->errstr );
}


=item B<DB_getdata>

Retrieve data from DB

Returns a reference to an array if called in scalar context, else returns an array. By default, the array (reference) contains several array-references that represent the rows returned by the statement. Each of these contains the values representing the selected columns.

Returns an array containing the selected values if ONECOLUMN is set to 1

B<Required Params:>
 STMT => 'sql-stmt' 
 DBH  => DBI-Object (Database Handle)

B<Optional Params:>
 FORMAT => [html|unicode] (convert results into 
                          specified format)
 ONECOLUMN => 0|1 (return simple array)

=cut

sub DB_getdata
{
	my %args = (
		STMT => '',
		DBH => '',
		FORMAT => '',
		ONECOLUMN => 0,
		@_
	);
   
	# dprint ("GET: STMT: $args{STMT}"); #  if DEBUG;
	my $ary_ref;

	if ($args{ONECOLUMN} == 1) {
    	$ary_ref = $args{DBH}->selectcol_arrayref($args{STMT}) || VocTrainer::Utils::handle_error ("DBH: $args{DBH}<br>STMT: $args{STMT}<br> ERROR: $args{DBH}->errstr");
    } else {
		my $sth = $args{DBH}->prepare($args{STMT}) || VocTrainer::Utils::handle_error ("DBH: $args{DBH}<br>STMT: $args{STMT}<br> ERROR: $args{DBH}->errstr");
	    my $rv = $sth->execute() || VocTrainer::Utils::handle_error ("DBH: $args{DBH}<br>STMT: $args{STMT}<br> ERROR: $args{DBH}->errstr");
    	$ary_ref = $sth->fetchall_arrayref([0..-1]);
	}

    if ($args{FORMAT}) {
		$ary_ref = VocTrainer::Utils::ascii2htmlform($ary_ref) if ($args{FORMAT} eq "htmlform");
		$ary_ref = VocTrainer::Utils::ascii2html($ary_ref) if ($args{FORMAT} eq "html");
		$ary_ref = VocTrainer::Utils::ascii2unicode($ary_ref) if ($args{FORMAT} eq "unicode");
    }

    return wantarray ? @{$ary_ref} : $ary_ref;
}


=item B<DB_insertdata>

Insert data into DB, returns nothing

B<Required Params:>
 STMT => 'sql-stmt' 
 DBH  => $DBI::Object (Database Handle)

=cut


sub DB_insertdata
{
	my %args = (DBH => '',
				STMT => '',
				@_
				);

	# print STDERR "INSERT: STMT: $args{STMT}\n"; #  if DEBUG; 
	return unless ($args{DBH} && $args{STMT});
    
    my $sth = $args{DBH}->prepare($args{STMT}) || VocTrainer::Utils::handle_error("ERROR: $args{DBH}->errstr STMT: $args{STMT}");
    my $rv = $sth->execute() || VocTrainer::Utils::handle_error($args{DBH}->errstr);

}

###
### Help Functions 
###

###########################################
### DB_getlastinsertid
### Select last insert id in current dbh
###########################################

=item B<DB_getlastinsertid>

Returns last insert id in current dbh


B<Required Params:>
 DBH  => $DBI::Object (Database Handle)

=cut


sub DB_getlastinsertid {
	my %args = (DBH => '',
				@_
				);

	my $stmt = "select LAST_INSERT_ID()";
    my @data = DB_getdata(
		STMT =>  $stmt,
		DBH => $args{DBH} ,
		ONECOLUMN => 1
			);
    return $data[0];
}


=item B<DB_begintrans>

Not used now

=cut


sub DB_begintrans
{

}


=item B<DB_rollback>

Not used now

=cut


sub DB_rollback
{
    my $dbh = $_[0];
    if ($conf{db} eq "oracle" || $conf{db} eq "sybase")
    {
        $dbh->rollback || VocTrainer::Utils::handle_error ("$dbh->errstr");
    }
    else
    {
	####
    }

}

=item B<DB_commit>

Not used now

=cut


sub DB_commit
{
    my $dbh = $_[0];
    if ($conf{db} eq "oracle" || $conf{db} eq "sybase")
    {
        $dbh->commit || VocTrainer::Utils::handle_error ("$dbh->errstr");
    }
    else
    {
	####
    }

}


=back

=head1 TODO

Wrapper for different SQL-Dialects (e.g. dateformats, limit, ...)

=head1 BUGS

Only tested with mysql.

=head1 SEE ALSO

L<DBI>, L<VocTrainer::Config>, L<VocTrainer::Utils> 

=head1 AUTHORS

slayven@xover.mud.at, sc@gimp.org, koppi@weirdoz.org

=cut

